package org.ox.ox_onekey_flutter;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class OxJsonUtil {
    private static String LOG_TAG = "OxJsonUtil";

    /**
     * str map parse JSONObject
     * If string form JSON is wrong, you get an object is null.
     *
     * @param obj string map
     * @return JSONObject
     */
    @Nullable
    public static JSONObject parseObject(Object obj) {
        JSONObject jsonObject = null;
        try {
            if (obj != null) {
                if (obj instanceof String) {
                    jsonObject = new JSONObject((String) obj);
                } else if (obj instanceof Map) {
                    jsonObject = new JSONObject((Map) obj);
                }
            }
        } catch (Exception e) {
        }
        return jsonObject;
    }


    /**
     * json string to map
     * @param strJson  json
     * @return Map<String, Object>
     */
    @Nullable
    public static Map<String, Object> jsonStringToMap(String strJson) {
        JSONObject obj = parseObject(strJson);
        try {
            if (obj != null) {
                return jsonObjectToMap(obj);
            }
        } catch (Exception e) {
        }
        return null;
    }

    /**
     * org.json.JSONObject  to Map
     *
     * @param json jsonObject
     * @return map
     * @throws JSONException
     */
    @NonNull
    public static Map<String, Object> jsonObjectToMap(JSONObject json) throws JSONException {
        Map<String, Object> result = new HashMap<String, Object>();
        if (json != null) {
            Iterator<String> keyIt = json.keys();
            while (keyIt.hasNext()) {
                String key = keyIt.next();
                Object val = json.get(key);
                if (val instanceof JSONObject) {
                    Map<String, Object> valMap = jsonObjectToMap((JSONObject) val);
                    result.put(key, valMap);
                } else if (val instanceof JSONArray) {
                    JSONArray ja = (JSONArray) val;
                    result.put(key, jsonArrayToList(ja));
                } else {
                    result.put(key, val);
                }
            }
        }
        return result;
    }

    /**
     * JSONArray to List<Object>
     * json array to list ,只能将jsonArray中的JSONObject 转化为Map，每一个实体类与map
     * 的形式转化；
     *
     * @param jsonArray jsonArray
     * @return List
     * @throws JSONException
     */
    public static List<Object> jsonArrayToList(JSONArray jsonArray) throws JSONException {
        List<Object> list = new ArrayList<Object>();
        if (jsonArray != null) {
            for (int i = 0; i < jsonArray.length(); i++) {
                Object val = jsonArray.get(i);
                if (val != null) {
                    if (val instanceof JSONObject) {
                        Map<String, Object> map = jsonObjectToMap((JSONObject) val);
                        list.add(map);
                    } else if (val instanceof JSONArray) {
                        list.add(jsonArrayToList((JSONArray) val));
                    } else {
                        list.add(val);
                    }
                }
            }
        }
        return list;
    }


}
