package org.ox.ox_onekey_flutter;

import android.app.Activity;
import android.app.Application;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import androidx.annotation.NonNull;
import androidx.annotation.UiThread;

import org.json.JSONObject;
import org.ox.base.OxActionType;
import org.ox.base.OxNotifier;
import org.ox.base.OxRequestParam;
import org.ox.base.OxRequestResult;
import org.ox.face.OxClientEntry;

import java.util.HashMap;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * OxOnekeyFlutterPlugin
 */
public class OxOnekeyFlutterPlugin implements FlutterPlugin, MethodCallHandler {

	private static LifecycleListener mDelegate = new LifecycleListener();

	@Override
	public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
		final MethodChannel channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "ox_onekey_flutter");
		channel.setMethodCallHandler(this);
		((Application) flutterPluginBinding.getApplicationContext()).unregisterActivityLifecycleCallbacks(mDelegate);
		((Application) flutterPluginBinding.getApplicationContext()).registerActivityLifecycleCallbacks(mDelegate);

	}

	public static void registerWith(Registrar registrar) {
		final MethodChannel channel = new MethodChannel(registrar.messenger(), "ox_onekey_flutter");
		channel.setMethodCallHandler(new OxOnekeyFlutterPlugin());
		((Application) registrar.activeContext().getApplicationContext()).unregisterActivityLifecycleCallbacks(mDelegate);
		((Application) registrar.activeContext().getApplicationContext()).registerActivityLifecycleCallbacks(mDelegate);

	}


	@Override
	public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
	}


	@Override
	public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
		String methodName = call.method;
		switch (methodName) {
			case "getSdkVersion":
				result.success(OxClientEntry.SDK_VERSION);
				break;
			case "oxInitSaiSDK": {
				initSdk(call, result);
			}
			break;
			case "oxNetworkInfo":
				oxNetworkInfo(result);
				break;
			case "oxOnekeyLogin":
				oxOnekeyLogin(result);
				break;
			case "oxMobileAuth":
				oxMobileAuth(result);
				break;
			case "oxPreLogin":
				oxPreLogin(result);
				break;
			case "oxDismissLoginPage":
				oxDismissLoginPage();
				break;
			default:
				result.notImplemented();
		}
	}

	private void oxNetworkInfo(Result result) {
		String networkInfo = OxClientEntry.getNetworkType();
		result.success(OxJsonUtil.jsonStringToMap(networkInfo));
	}

	private void initSdk(@NonNull MethodCall call, @NonNull Result callResult) {
		Object arguments = call.arguments;
		String jsonInitParam = null;
		if (arguments instanceof Map) {
			jsonInitParam = new JSONObject((Map) arguments).toString();
		} else if (arguments instanceof JSONObject) {
			jsonInitParam = arguments.toString();
		}
		OxRequestParam param = new OxRequestParam();
		param.setActionType(OxActionType.INIT_SDK);
		param.setStrData(jsonInitParam);
		param.setExtra(callResult);
		OxClientEntry.setDefaultTimeout(8000);
		OxClientEntry.setDebugMode(true);
		int iResult = OxClientEntry.init(mDelegate.getCurrentActivity(), param, new MyNotifier());

		if (iResult < 0) {
			callResult.error(iResult + "", "sdk 初始化失败", null);
		}
	}

	private void oxOnekeyLogin(Result pResult) {
		OxRequestParam param = new OxRequestParam();
		param.setActionType(OxActionType.GET_ONEKEY_LOGIN_TOKEN);
		param.setExtra(pResult);
		int iResult = OxClientEntry.requestAction(param);
		if (iResult < 0) {
			Map<String, Object> iMap = new HashMap<>();
			iMap.put("code", iResult);
			Map<String, Object> call = new HashMap<>();
			call.put("result", iMap);
			call.put("action_type", OxActionType.GET_ONEKEY_LOGIN_TOKEN);
			pResult.success(call);
		}
	}

	private void oxMobileAuth(Result pResult) {
		OxRequestParam param = new OxRequestParam();
		param.setActionType(OxActionType.GET_MOBILE_AUTH_TOKEN);
		param.setExtra(pResult);
		int iResult = OxClientEntry.requestAction(param);
		if (iResult < 0) {
			Map<String, Object> iMap = new HashMap<>();
			iMap.put("code", iResult);
			Map<String, Object> call = new HashMap<>();
			call.put("action_type", OxActionType.GET_MOBILE_AUTH_TOKEN);
			call.put("result", iMap);
			pResult.success(call);
		}
	}

	private void oxPreLogin(Result pResult) {
		OxRequestParam param = new OxRequestParam();
		param.setActionType(OxActionType.PRE_LOGIN);
		param.setExtra(pResult);
		int iResult = OxClientEntry.requestAction(param);
		if (iResult < 0) {
			Map<String, Object> iMap = new HashMap<>();
			iMap.put("code", iResult);
			Map<String, Object> call = new HashMap<>();
			call.put("result", iMap);
			pResult.success(call);
		}
	}

	private void oxDismissLoginPage() {
		OxClientEntry.finishAuthActivity();
	}

	private static class MyNotifier implements OxNotifier {
		@UiThread
		@Override
		public void onActionResult(final OxRequestResult result) {
			new Handler(Looper.getMainLooper()).post(new Runnable() {
				@Override
				public void run() {
					try {
						if (result.getParam().getExtra() != null) {
							Result callResult = (Result) result.getParam().getExtra();
							Map<String, Object> requestMap = OxJsonUtil.jsonStringToMap(result.getStrData());
							Map<String, Object> call = new HashMap<>();
							call.put("result", requestMap);
							call.put("action_type", result.getActionType());
							callResult.success(call);
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			});
		}
	}

	private static class LifecycleListener implements Application.ActivityLifecycleCallbacks {
		private Activity mCurrentActivity;

		private Activity getCurrentActivity() {
			return mCurrentActivity;
		}

		@Override
		public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
			mCurrentActivity = activity;
		}

		@Override
		public void onActivityStarted(Activity activity) {
			mCurrentActivity = activity;
		}

		@Override
		public void onActivityResumed(Activity activity) {
			mCurrentActivity = activity;
		}

		@Override
		public void onActivityPaused(Activity activity) {
			mCurrentActivity = activity;
		}

		@Override
		public void onActivityStopped(Activity activity) {
			mCurrentActivity = activity;
		}

		@Override
		public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
			mCurrentActivity = activity;
		}

		@Override
		public void onActivityDestroyed(Activity activity) {
			mCurrentActivity = activity;
		}
	}

}
